/*
       B R O C C O L I  --  The Bro Client Communications Library

Copyright (C) 2004-2005 Christian Kreibich <christian.kreibich(at)cl.cam.ac.uk>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
#ifndef broccoli_val_h
#define broccoli_val_h

#include <broccoli.h>
#include <bro_types.h>
#include <bro_util.h>
#include <bro_id.h>
#include <bro_attrs.h>

typedef void *(* BroValAccessor) (BroVal *val);

struct bro_val
{
  BroObject                    object;

  /* A class method for accessing the data contained by
   * a val:
   */
  BroValAccessor               get_data;
  
  /* Vals can be chained together as a list here: */
  TAILQ_ENTRY(bro_val)         val_list;

  /* The full type object of this val -- this also yields
   * what member of the union is used below (unless we're
   * inside a derived type and store our data elsewhere).
   * If val_type is NULL, it means we're dealing with an
   * unassigned val.
   */
  BroType                     *val_type;
  
  BroAttrs                    *val_attrs;

  union {
    uchar                      char_val;
    uint32                     int_val;
    double                     double_val;
    BroPort                    port_val;
    BroString                  str_val;
    BroSubnet                  subnet_val;
  } val;

#define val_char               val.char_val
#define val_int                val.int_val
#define val_double             val.double_val
#define val_port               val.port_val
#define val_str                val.str_val
#define val_strlen             val.str_val.str_len
#define val_subnet             val.subnet_val
};

struct bro_mutable_val
{
  BroVal                       val;

  BroID                       *id;
  char                         props;
};

#define BRO_VAL_PROP_PERS      0x01
#define BRO_VAL_PROP_SYNC      0x02

struct bro_record_val
{
  BroMutableVal                mutable;
  
  /* We don't use the full record val when interacting
   * with the user, but only what's really necessary:
   */
  BroRecord                   *rec;
};

BroVal          *__bro_val_new(void);
BroVal          *__bro_val_new_of_type(int type);
int              __bro_val_assign(BroVal *val, const void *data);

/* Returns a pointer to the val's data depending on its type.
 * Type is returned through *type if provided.
 */
int              __bro_val_get_data(BroVal *val, int *type, void **data);
int              __bro_val_get_type_num(const BroVal *val);

void             __bro_val_init(BroVal *val);
void             __bro_val_free(BroVal *val);
int              __bro_val_read(BroVal *val, BroConn *bc);
int              __bro_val_write(BroVal *val, BroConn *bc);
int              __bro_val_clone(BroVal *dst, BroVal *src);
void            *__bro_val_get(BroVal *val);

BroMutableVal   *__bro_mutable_val_new(void);
void             __bro_mutable_val_init(BroMutableVal *mv);
void             __bro_mutable_val_free(BroMutableVal *mv);
int              __bro_mutable_val_read(BroMutableVal *mv, BroConn *bc);
int              __bro_mutable_val_write(BroMutableVal *mv, BroConn *bc);
int              __bro_mutable_val_clone(BroMutableVal *dst, BroMutableVal *src);

BroRecordVal    *__bro_record_val_new(void);
void             __bro_record_val_init(BroRecordVal *rv);
void             __bro_record_val_free(BroRecordVal *rv);
int              __bro_record_val_read(BroRecordVal *rv, BroConn *bc);
int              __bro_record_val_write(BroRecordVal *rv, BroConn *bc);
int              __bro_record_val_clone(BroRecordVal *dst, BroRecordVal *src);
void            *__bro_record_val_get(BroRecordVal *rv);

#endif
